@@ -108,14 +108,11 @@ __PACKAGE__->meta->make_immutable;
 
 =head1 AUTHOR
 
-Johannes Plunien E<lt>plu@cpan.orgE<gt>
+See L<Gearman::Driver>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2009 by Johannes Plunien
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+See L<Gearman::Driver>.
 
 =head1 SEE ALSO
 
@@ -129,6 +126,8 @@ it under the same terms as Perl itself.
 
 =item * L<Gearman::Driver::Job>
 
+=item * L<Gearman::Driver::Loader>
+
 =item * L<Gearman::Driver::Worker>
 
 =back
