@@ -49,8 +49,6 @@ Gearman::Driver::Worker - Base class for workers
 
     1;
 
-=head1 DESCRIPTION
-
 =head1 ATTRIBUTES
 
 =head2 server
@@ -270,20 +268,19 @@ you want to Encode/Decode all your jobs:
         my ( $self, $job, $workload ) = @_;
     }
 
+=cut
+
 no Moose;
 
 __PACKAGE__->meta->make_immutable;
 
 =head1 AUTHOR
 
-Johannes Plunien E<lt>plu@cpan.orgE<gt>
+See L<Gearman::Driver>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2009 by Johannes Plunien
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+See L<Gearman::Driver>.
 
 =head1 SEE ALSO
 
@@ -297,8 +294,14 @@ it under the same terms as Perl itself.
 
 =item * L<Gearman::Driver::Job>
 
+=item * L<Gearman::Driver::Loader>
+
 =item * L<Gearman::Driver::Observer>
 
+=item * L<Gearman::Driver::Worker::AttributeParser>
+
+=item * L<Gearman::Driver::Worker::Base>
+
 =back
 
 =cut
