@@ -1,5 +1,4 @@
-package    # hide from PAUSE
-  Loader::Empty;
+package Loader::Empty;
 
 use Moose;
 with qw(Gearman::Driver::Loader);
